# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

import os
import logging

from covalent_api import url_utils as utils
import covalent_api

logging.basicConfig()
logger = logging.getLogger('Tests')
logger.setLevel(logging.INFO)



args_dict = {
    'chain_id': '1',
    'address': '0x74c1e4b8cae59269ec1d85d3d4f324396048f4ac',
    'block_height': '1',
    'starting_block': '1',
    'ending_block': '3',
    'topic': '0x8c5be1e5ebec7d5bd14f71427d1e84f3dd0314c0f7b2291e5b200ac8c7c3b925',
    'token_id': '123',
    'id': 'all',
    'tx_hash': '0x77cff4d41d028c10f5fa1ecbc8c899ce537332e9833d4477bbde110e7bc5b5e6',
    'quote_currency': 'EUR',
    'contract_address': '0xB8c77482e45F1F44dE1745F52C74426C631bDD52',
    'contract_addresses': '0xB8c77482e45F1F44dE1745F52C74426C631bDD52',
    'ticker_symbol': 'ETH'
}

#Test class methods
def test_class(cls, class_methods):
    log_errors = []

    for meth_name in class_methods:
        meth = getattr(cls, meth_name)
        required_args, optional_arguments = utils.get_method_arguments(meth)
        meth_args = {}
        for k, v in args_dict.items():
            if k in required_args:
                meth_args[k] = v
        if meth_name == 'get_all_contract_metadata':
            meth_args['chain_id'] = '56'
        try:
            result = meth(**meth_args)
        except Exception as e:
            raise Exception(
                "Could not execute the method {}. \n Arguments: {} \n Error: {}".format(
                    meth_name, meth_args, e
                )
            )

        if not result:
            log_errors.append({'method_name': meth_name, 'result': result})
            logger.debug("meth: {}".format(meth_name))
            logger.debug("result: {}".format(result))
            logger.debug("Test Error")
            logger.debug("******************************")
            continue
        if isinstance(result, dict):
            if result.get('error'):
                log_errors.append({'method_name': meth_name, 'result': result})
                logger.debug("meth: {}".format(meth_name))
                logger.debug("result: {}".format(result))
                logger.debug("Test Error")
                logger.debug("******************************")
                continue
        else:
            log_errors.append({'method_name': meth_name, 'result': result})
            logger.debug("meth: {}".format(meth_name))
            logger.debug("result: {}".format(result))
            logger.debug("Test Error")
            logger.debug("******************************")
            continue

        logger.debug("meth: {}".format(meth_name))
        logger.debug("result: {}".format(result))
        logger.debug("Test OK")
        logger.debug("******************************")

    print("Class {} test result: ".format(cls))
    if log_errors:
        print("The following errors found:")
        for log_error in log_errors:
            print(
                "{}:\n result: {}".format(
                    log_error.get('method_name'),
                    log_error.get('result')
                )
            )
    else:
        print("No errors found")

    print("################################# \n")
    return log_errors


# Create session
session = covalent_api.Session(
    server_url='https://api.covalenthq.com',
    api_key=os.environ.get('COVALENT_API_KEY')
)

#Test class a
class_a = covalent_api.ClassA(session)
class_a_methods = utils.get_class_methods(covalent_api.ClassA)
errors_a = test_class(class_a, class_a_methods)

#Test class b
class_b = covalent_api.ClassB(session)
class_b_methods = utils.get_class_methods(covalent_api.ClassB)
errors_b = test_class(class_b, class_b_methods)

#Test class pricing
class_prices = covalent_api.Pricing(session)
class_prices_methods = utils.get_class_methods(covalent_api.Pricing)
errors_prices = test_class(class_prices, class_prices_methods)



# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

import os
import json
import logging
import requests


class Session(object):
    '''Attach session authentication information to requests.'''

    @property
    def server_url(self):
        '''Return server url used for session.'''
        return self._server_url

    @property
    def api_key(self):
        '''Return API key used for session.'''
        return self._api_key

    def __init__(
            self, server_url='https://api.covalenthq.com',
            api_key=None,
            timeout=60
    ):
        '''
        Initialize Session with the given *server_url* and *api_key*

        :param server_url: Default covalent api url: https://api.covalenthq.com
            If None, will check for envar:: COVALENT_SERVER environment variable.
        :type server_url: string
        :param api_key: Personal covalent API key. If None check for
        envar:: COVALENT_API_KEY environment variable.
        :type api_key: string
        :param timeout: Timeout in case the server is not responding.
        :type timeout: float
        '''
        super(Session, self).__init__()
        logging.basicConfig()

        self.logger = logging.getLogger(
            __name__ + '.' + self.__class__.__name__
        )
        self.logger.setLevel(logging.INFO)

        #If no server_url provided check the environment variable.
        if server_url is None:
            server_url = os.environ.get('COVALENT_SERVER')

        if not server_url:
            raise TypeError(
                'Required "server_url" not specified. Pass as argument or set '
                'in environment variable COVALENT_SERVER.'
            )

        self._server_url = server_url

        #If no api_key provided check the environment variable.
        if api_key is None:
            api_key = os.environ.get('COVALENT_API_KEY')

        if not api_key:
            raise TypeError(
                'Required "api_key" not specified. Pass as argument or set in '
                'environment variable COVALENT_API_KEY.'
            )

        self._api_key = api_key

        self._request = requests.Session()
        self._request.auth = ('', self._api_key)

        self.request_timeout = timeout

    def decode(self, string):
        '''
        Return decoded JSON *string* as Python object.
        '''
        return json.loads(string)

    def encode(self, data):
        '''Return encoded *data* as JSON'''
        return json.dumps(
            data,
            sort_keys=True
        )

    def _check_params(self, params):
        '''
        Check given *params* to remove invalid ones.
        :param params: Dictionary with parameters.
        :type params: dictionary
        '''
        if not isinstance(params, dict):
            raise Exception("params should be a dicctionary")
        params_to_pop = []
        for k, v in params.items():
            if v==None:
                params_to_pop.append(k)
        for param in params_to_pop:
            params.pop(param)
        return params

    def query(self, url, params=None, decode=True):
        '''
        Query the *url* request with the given *params*

        :param url: path url to query.
        :type url: string
        :param params: Dictionary with url parameters
        :type params: dictionary
        :param decode: Json decode the returned response from the server.
            True by default.
        :type decode: boolean
        '''
        url = "{}{}".format(self._server_url, url)


        self.logger.debug("Url: {}".format(url))
        #params = self.encode(params)
        params = self._check_params(params)

        self.logger.debug("Parameters: {}".format(params))

        response = self._request.get(
            url,
            params=params,
            timeout=self.request_timeout
        )

        result = response.text
        if not result:
            return result
        if decode:
            try:
                result = self.decode(result)
            except Exception as e:
                error_message = (
                    'Server reported error in unexpected format. '
                    'Raw error was: {0}. \n Exception: {1}'.format(response.text, e)
                )
                self.logger.exception(error_message)
                raise Exception(error_message)

        return result




# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

from covalent_api.constants.maps import *


# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

#Available chains id for historical prices, probably for all the queries too.
AVAILABLE_CHAIN_IDS = {
    'Ethereum Mainnet': '1',
    'Polygon/Matic Mainnet': '137',
    'Polygon/Matic Mumbai Testnet': '80001',
    'Binance Smart Chain': '56',
    'Avalanche C-Chain Mainnet': '43114',
    'Fuji C-Chain Testnet': '43113',
    'Fantom Opera Mainnet': '250'
}

#Available chains id contract metadata
AVAILABLE_CHAIN_IDS_CONTRACT_METADATA = {
    'Polygon/Matic Mainnet': '137',
    'Binance Smart Chain': '56',
    'Avalanche C-Chain Mainnet': '43114',
    'Fuji C-Chain Testnet': '43113',
    'Fantom Opera Mainnet': '250'
}


# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

from covalent_api import constants


class ClassB(object):

    @property
    def session(self):
        return self._session

    def __init__(self, session):
        self._session = session

    def get_sushiswap_address_exchange_liquidity_transactions(
            self, chain_id, address, swaps=False
    ):
        '''
        Get Sushiswap address exchange liquidity transactions.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically.
        :type address: string
        :param swaps: Get additional insight on swap event data related to this
            address, default: false
        :type swaps: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/address/{address}/stacks/sushiswap/acts/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'swaps': swaps
        }

        result = self.session.query(method_url, params, decode=True)
        return result


    def get_sushiswap_address_exchange_balances(self, chain_id, address):
        '''
        Get Sushiswap address exchange balances. Passing in an ENS resolves
        automatically.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/address/{address}/stacks/sushiswap/balances/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_sushiswap_network_assets(self, chain_id):
        '''
        Return a paginated list of Sushiswap pools sorted by exchange volume.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = '/v1/{chain_id}/networks/sushiswap/assets/'.format(
                chain_id=chain_id
            )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_aave_v2_address_balances(self, address):
        '''
        Get Aave v2 address balances, supply and borrow positions.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = (
            '/v1/1/address/{address}/stacks/aave_v2/balances/'.format(
                address=address
            )
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_aave_address_balances(self, address):
        '''
        Get Aave address balances.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = '/v1/1/address/{address}/stacks/aave/balances/'.format(
            address=address
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_balancer_exchange_address_balances(self, address):
        '''
        Get Balancer exchange address balances.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = '/v1/1/address/{address}/stacks/balancer/balances/'.format(
            address=address
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_compound_address_activity(self, address):
        '''
        Get Compound address activity.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = '/v1/1/address/{address}/stacks/compound/acts/'.format(
            address=address
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_compound_address_balances(self, address):
        '''
        Get Compound address balances.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = '/v1/1/address/{address}/stacks/compound/balances/'.format(
            address=address
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_curve_address_balances(self, address):
        '''
        Get Curve address balances.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = '/v1/1/address/{address}/stacks/curve/balances/'.format(
            address=address
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_farming_stats(self, address):
        '''
        Get farming positions on Uniswap, Sushiswap, and Harvest.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = '/v1/1/address/{address}/stacks/farming/positions/'.format(
            address=address
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_marker_address_balances_and_acts(self, address):
        '''
        Return user activity in the MakerDAO protocol.

        Inspired by this blog post:
        https://www.covalenthq.com/blog/defi-data-availability-makerdao-csv-export/

        Some unique features:
        1) Return all backing collateral types
        2) Return vaults belonging to other sub-wallets like InstaDapp
        3) Return SHUT/CLOSED/LIQUIDATED Vaults

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = '/v1/1/address/{address}/stacks/makerdao/'.format(
            address=address
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_uniswap_v1_address_exchange_balances(self, address):
        '''
        Get Uniswap v1 address exchange balances.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = (
            '/v1/1/address/{address}/stacks/uniswap_v1/balances/'.format(
                address=address
            )
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_uniswap_v2_address_exchange_liquidity_transactions(
            self, address, swaps=False
    ):
        '''
        Get Uniswap v2 address exchange liquidity transactions.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        :param swaps: Get additional insight on swap event data related to this
            address, default: false
        :type swaps: string
        '''
        method_url = '/v1/1/address/{address}/stacks/uniswap_v2/acts/'.format(
            address=address
        )
        params = {
            'swaps': swaps
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_uniswap_v2_address_exchange_balances(self, address):
        '''
        Get Uniswap v2 address exchange balances. Passing in an ENS resolves
        automatically.

        :param address: Passing in an ENS resolves automatically.
        :type address: string
        '''
        method_url = '/v1/1/address/{address}/stacks/uniswap_v2/balances/'.format(
            address=address
        )
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_aave_v2_network_assets(self):
        '''
        Get Aave v2 network assets.
        '''
        method_url = '/v1/1/networks/aave_v2/assets/'
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_aave_network_assets(self):
        '''
        Get Aave network assets.
        '''
        method_url = '/v1/1/networks/aave/assets/'
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_augur_market_affiliate_fee_divisors(
            self, page_number=None, page_size=None
    ):
        '''
        Get Augur market affiliate fee divisors.

        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        '''
        method_url = '/v1/1/networks/augur/affiliate_fee/'
        params = {
            'page-number':page_number,
            'page-size':page_size
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_compound_network_assets(self):
        '''
        Get Compound network assets.
        '''
        method_url = '/v1/1/networks/compound/assets/'
        params = {
        }

        result = self.session.query(method_url, params, decode=True)
        return result

    def get_uniswap_v2_network_assets(
            self, tickers=None, page_number=None, page_size=None
    ):
        '''
        Return a paginated list of Uniswap pools sorted by exchange volume.

        :param tickers: If tickers (a comma separated list) is present, only
            return the pools that contain these tickers.
        :type tickers: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        '''
        method_url = '/v1/1/networks/uniswap_v2/assets/'
        params = {
            'tickers': tickers,
            'page-number':page_number,
            'page-size':page_size
        }

        result = self.session.query(method_url, params, decode=True)
        return result


# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.


from covalent_api import constants


class Pricing(object):

    @property
    def session(self):
        return self._session

    def __init__(self, session):
        self._session = session

    def get_prices_by_address(
            self, chain_id, quote_currency, contract_address, date_from=None,
            date_to=None, prices_at_asc=False, page_number=None, page_size=None,
            format="json"
    ):
        '''
        Get historical prices for a contract_address in a particular chain_Id
        and quote_currency. Can pass to and from to define a range, by default
        if they are omitted, it returns today's price.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param quote_currency: The requested fiat currency.
        :type quote_currency: string
        :param contract_address: Smart contract address.
        :type contract_address: string
        :param date_from: The start day of the historical price range. (YYYY-MM-DD)
        :type date_from: string
        :param date_to: The end day of the historical price range. (YYYY-MM-DD)
        :type date_to: string
        :param prices_at_asc: Sort the prices in chronological ascending order.
            By default, it's set to false and returns prices in chronological
            descending order.
        :type prices_at_asc: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses.
        :type format: string
        '''
        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/pricing/historical_by_address/{chain_id}/{quote_currency}/'
            '{contract_address}/'.format(
                chain_id=chain_id,
                quote_currency=quote_currency,
                contract_address=contract_address
            )
        )
        params = {
            'from': date_from,
            'to': date_to,
            'prices-at-asc': prices_at_asc,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'
        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_prices_by_addresses(
            self, chain_id, quote_currency, contract_addresses, date_from=None,
            date_to=None, prices_at_asc=False, page_number=None, page_size=None,
            format="json"
    ):
        '''
        Get historical prices for a contract_address, or a comma-separated group
        of contract_addresses in a particular chain_id and quote_currency.
        Can pass to and from to define a range, by default if they are omitted,
        it returns today's price.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param quote_currency: The requested fiat currency.
        :type quote_currency: string
        :param contract_addresses: Smart contract address(es).
        :type contract_addresses: string
        :param date_from: The start day of the historical price range. (YYYY-MM-DD)
        :type date_from: string
        :param date_to: The end day of the historical price range. (YYYY-MM-DD)
        :type date_to: string
        :param prices_at_asc: Sort the prices in chronological ascending order.
            By default, it's set to false and returns prices in chronological
            descending order.
        :type prices_at_asc: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses.
        :type format: string
        '''
        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/pricing/historical_by_addresses/{chain_id}/{quote_currency}/'
            '{contract_addresses}/'.format(
                chain_id=chain_id,
                quote_currency=quote_currency,
                contract_addresses=contract_addresses
            )
        )
        params = {
            'from': date_from,
            'to': date_to,
            'prices-at-asc': prices_at_asc,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'
        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_historical_prices_by_ticker_symbol(
            self, quote_currency, ticker_symbol, date_from=None, date_to=None,
            prices_at_asc=False, page_number=None, page_size=None, format="json"
    ):
        '''
        Get historical prices for a ticker_symbol in a particular quote_currency.
        Can pass to and from to define a range, by default if they are omitted,
        it returns today's price.

        :param quote_currency: The requested fiat currency.
        :type quote_currency: string
        :param ticker_symbol:
        :type ticker_symbol: string
        :param date_from: The start day of the historical price range. (YYYY-MM-DD)
        :type date_from: string
        :param date_to: The end day of the historical price range. (YYYY-MM-DD)
        :type date_to: string
        :param prices_at_asc: Sort the prices in chronological ascending order.
            By default, it's set to false and returns prices in chronological
            descending order.
        :type prices_at_asc: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses.
        :type format: string
        '''

        method_url = (
            '/v1/pricing/historical/{quote_currency}/{ticker_symbol}/'.format(
                quote_currency=quote_currency, ticker_symbol=ticker_symbol
            )
        )
        params = {
            'from': date_from,
            'to': date_to,
            'prices-at-asc': prices_at_asc,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'
        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_spot_prices(
            self, tickers=None, page_number=None, page_size=None, format="json"
    ):
        '''
        Get spot prices and metadata for all tickers or a select group of tickers.
        Without tickers query param, it returns a paginated list of all tickers
        sorted by market cap.

        :param tickers: If tickers (a comma separated list of tickers is present),
            only return the spot prices for these tokens.
        :type tickers: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses.
        :type format: string
        '''
        method_url = '/v1/pricing/tickers/'
        params = {
            'tickers': tickers,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'
        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_price_volatility(
            self, tickers=None, page_number=None, page_size=None, format="json"
    ):
        '''
        Get price volatility and metadata for a select group of tickers.
        Without the tickers query param, it defaults to ETH volatility.

        :param tickers: If tickers (a comma separated list of tickers is present),
            only return the spot prices for these tokens.
        :type tickers: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses.
        :type format: string
        '''
        method_url = '/v1/pricing/volatility/'
        params = {
            'tickers': tickers,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'
        result = self.session.query(method_url, params, decode=decode)
        return result



# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.


from covalent_api import constants


class ClassA(object):

    @property
    def session(self):
        return self._session

    def __init__(self, session):
        self._session = session

    def get_token_balances_for_address(
            self, chain_id, address, nft=False, no_nft_fetch=False, format="json"
    ):
        '''
        Return a list of all ERC20 and NFT token balances along with their
        current spot prices.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string
        :param address: Passing in an ENS resolves automatically.
        :type address: string
        :param nft: Set to true to return ERC721 and ERC1155 assets. Defaults to
            false.
        :type nft: boolean
        :param no_nft_fetch: Set to true to skip fetching NFT metadata, which
            can result in faster responses. Defaults to false and only applies when
            nft=true.
        :type no_nft_fetch: boolean
        :param format: If format=csv, return a flat CSV instead of JSON responses.
        :type format: string
        '''
        # TODO: on the api docs says the following for the address argument:
        #  Passing in an ENS resolves automatically. But I don't know what an
        #  ENS is.
        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/address/{address}/balances_v2/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'nft': nft,
            'no-nft-fetch': no_nft_fetch,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_historical_portfolio_value_over_time(
            self, chain_id, address, quote_currency=None, format="json"
    ):
        '''
        Given chain_id and wallet address, return wallet value for the last 30
        days at 24 hour timestamps.
        
        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string
        :param address: Passing in an ENS resolves automatically.
        :type address: string
        :param quote_currency: The requested fiat currency.
        :type quote_currency: string
        :param format: If format=csv, return a flat CSV instead of JSON responses.
        :type format: string
        '''
        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/address/{address}/portfolio_v2/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'quote-currency': quote_currency,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_transactions(
            self, chain_id, address, block_signed_at_asc=False, no_logs=False,
            page_number=None, page_size=None, format="json"
    ):
        '''
        Retrieve all transactions for address including their decoded log events.
        This endpoint does a deep-crawl of the blockchain to retrieve all kinds
        of transactions that references the address.
        Take for example the address: 0x5a6d3b6bf795a3160dc7c139dee9f60ce0f00cae,
        the endpoint returns the following transactions:

        Description                 Example
        Failed transactions         0x5545...da6c
        ETH-only value transfers    0x887f...93d0
        ERC20 transfer              0xe936...ead7


        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically. 
        :type address: string 
        :param block_signed_at_asc: Sort the transactions in chronological
            ascending order. By default, it's set to false and returns transactions
            in chronological descending order.
        :type block_signed_at_asc: bool
        :param no_logs: Setting this to true will omit decoded event logs, 
            resulting in lighter and faster responses. By default it's set to false.
        :type no_logs: boolean
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string 
         
         
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/address/{address}/transactions_v2/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'block-signed-at-asc': block_signed_at_asc,
            'no-logs': no_logs,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_erc20_token_transfers(
            self, chain_id, address, contract_address,
            page_number=None, page_size=None, format="json"
    ):
        '''
        Get ERC20 token transfers. Passing in an ENS resolves automatically.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically. 
        :type address: string 
        :param contract_address: 
        :type contract_address: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/address/{address}/transfers_v2/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'contract-address': contract_address,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_a_block(
            self, chain_id, block_height, format="json"
    ):

        '''
        Retrieve a single block at block_height. If block_height is set to the
        value latest, return the latest block available.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string
        :param block_height:
        :type block_height: string
        :param format: If format=csv, return a flat CSV instead of JSON responses.
        :type format: string
        '''
        
        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/block_v2/{block_height}/'.format(
                chain_id=chain_id,
                block_height=block_height
            )
        )
        params = {
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_log_events_by_contract_address(
            self, chain_id, address, starting_block, ending_block,
            page_number=None, page_size=None, format="json"
    ):
        '''
        Return a paginated list of decoded log events emiited by a particular
        smart contract.

        Here's an example of retrieving all Compound Governance events along
        with their decoded data, including proposals created/executed, votes cast,
        and so on: https://gist.github.com/gane5h/a117f507879e6a70bef1cc4240f3264d

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically. 
        :type address: string 
        :param starting_block: Starting block to define a block range.
        :type starting_block: string
        :param ending_block: Ending block to define a block range. Passing in
            'latest' uses the latest block height.
        :type ending_block: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )


        method_url = (
            '/v1/{chain_id}/events/address/{address}/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'starting-block': starting_block,
            'ending-block': ending_block,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_log_events_by_topic_hash(
            self, chain_id, topic, starting_block, ending_block, sender_address=None,
            page_number=None, page_size=None, format="json"
    ):
        '''
        Return a paginated list of decoded log events with one or more topic
        hashes separated by a comma.

        Check out our topic hash calculator here:
        https://www.covalenthq.com/docs/tools/topic-calculator

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param topic: 
        :type topic: string
        :param starting_block: Starting block to define a block range.
        :type starting_block: string
        :param ending_block: Ending block to define a block range. Passing in
            'latest' uses the latest block height.
        :type ending_block: string
        :param sender_address: The address of the sender.
        :type sender_address: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/events/topics/{topic}/'.format(
                chain_id=chain_id,
                topic=topic
            )
        )
        params = {
            'sender-address': sender_address,
            'starting-block': starting_block,
            'ending-block': ending_block,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_external_nft_metadata(
            self, chain_id, address, token_id, format="json"
    ):
        '''
        Given a NFT contract address and a token ID, fetch and return the
        external metadata.

        Both ERC751 as well as ERC1155 are supported.

        Example: https://gist.github.com/gane5h/e0e5f9ae2517d353324465d9cbe64ae9

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically. 
        :type address: string 
        :param token_id: The ID to the token
        :type token_id: string
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/tokens/{address}/nft_metadata/{token_id}/'.format(
                chain_id=chain_id,
                address=address,
                token_id=token_id
            )
        )
        params = {
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_nft_token_ids(
            self, chain_id, address, page_number=None, page_size=None,
            format="json"
    ):
        '''
        Returns a list of all token IDs for a NFT contract on a blockchain network.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically. 
        :type address: string 
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/tokens/{address}/nft_token_ids/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_nft_transactions(
            self, chain_id, address, token_id, page_number=None, page_size=None,
            format="json"
    ):
        '''
        Returns a list of transactions given a NFT contract and a token ID on a
        blockchain network.

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically. 
        :type address: string 
        :param token_id: The ID to the token
        :type token_id: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string 
         
         
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/tokens/{address}/nft_transactions/{token_id}/'.format(
                chain_id=chain_id,
                address=address,
                token_id=token_id
            )
        )
        params = {
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_changes_in_token_holders_between_two_block_heights(
            self, chain_id, address, starting_block, ending_block=None,
            page_number=None, page_size=None, format="json"
    ):
        '''
        Get token balance changes for token holders between starting-block and
        ending-block Return a paginated list of token holders and their
        current/historical balances. If ending-block is omitted, the latest
        block is used.

        Here's an example response for the UNI token between blocks 11000000
        and 11383362:
        https://gist.github.com/gane5h/23e2efcdeefb661fa2907fbbe2c81d5c

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically. 
        :type address: string 
        :param starting_block: Starting block to define a block range.
        :type starting_block: string
        :param ending_block: Ending block to define a block range. Passing in
            'latest' uses the latest block height.
        :type ending_block: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/tokens/{address}/token_holders_changes/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'starting-block': starting_block,
            'ending-block': ending_block,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_token_holders_as_of_a_block_height(
            self, chain_id, address, block_height=None,
            page_number=None, page_size=None, format="json"
    ):
        '''
        Return a paginated list of token holders. If block-height is omitted,
        the latest block is used.

        Here's an example response of the token holders of the UNI token as of
        block height 11383235:
        https://gist.github.com/gane5h/0c7c088c28def63e60cf7bd3957ce9a5

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param address: Passing in an ENS resolves automatically. 
        :type address: string 
        :param block_height: 
        :type block_height: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string
        '''

        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/tokens/{address}/token_holders/'.format(
                chain_id=chain_id,
                address=address
            )
        )
        params = {
            'block-height': block_height,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_all_contract_metadata(
            self, chain_id, id, page_number=None, page_size=None, format="json"
    ):
        '''
        Returns a list of all contracts on a blockchain along with their metadata.

        :param chain_id: Chain ID of the Blockchain being queried.
            Supports 56 for BSC Mainnet and 137 for Polygon/Matic Mainnet,
            43114 for Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet,
            and 250 for Fantom Opera Mainnet.
        :type chain_id: string 
        :param id: Only all supported right now.
        :type id: string
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string 
         
         
        '''
        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS_CONTRACT_METADATA.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS_CONTRACT_METADATA.values())
                )
            )
        if id != 'all':
            raise Exception(
                "Only all is supported right now"
            )

        method_url = (
            '/v1/{chain_id}/tokens/tokenlists/{id}/'.format(
                chain_id=chain_id,
                id=id
            )
        )
        params = {
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result

    def get_a_transaction(
            self, chain_id, tx_hash, no_logs=False, page_number=None,
            page_size=None, format="json"
    ):
        '''

        Retrieve a single transaction for tx_hash including their decoded log events.

        Here's an example response with fully decoded event logs:
        https://gist.github.com/gane5h/887ab9f81a364b804112cf1dee3cf7fc

        :param chain_id: Chain ID of the Blockchain being queried. Currently
            supports 1 for Ethereum Mainnet, 137 for Polygon/Matic Mainnet, 80001
            for Polygon/Matic Mumbai Testnet, 56 for Binance Smart Chain, 43114 for
            Avalanche C-Chain Mainnet, 43113 for Fuji C-Chain Testnet, and 250 for
            Fantom Opera Mainnet.
        :type chain_id: string 
        :param tx_hash: 
        :type tx_hash: string
        :param no_logs: Setting this to true will omit decoded event logs,
            resulting in lighter and faster responses. By default it's set to false.
        :type no_logs: boolean
        :param page_number: The specific page to be returned.
        :type page_number: int32
        :param page_size: The number of results per page.
        :type page_size: int32
        :param format: If format=csv, return a flat CSV instead of JSON responses. 
        :type format: string

        '''
        if chain_id not in list(constants.AVAILABLE_CHAIN_IDS.values()):
            raise Exception(
                "chain_id should be one of {}".format(
                    list(constants.AVAILABLE_CHAIN_IDS.values())
                )
            )

        method_url = (
            '/v1/{chain_id}/transaction_v2/{tx_hash}/'.format(
                chain_id=chain_id,
                tx_hash=tx_hash
            )
        )
        params = {
            'no-logs': no_logs,
            'page-number': page_number,
            'page-size': page_size,
            'format': format
        }
        decode = format == 'json'

        result = self.session.query(method_url, params, decode=decode)
        return result



# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

from covalent_api.session import Session
from covalent_api.pricing import *
from covalent_api.class_a import *
from covalent_api.class_b import *


# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

import os

from setuptools import setup, find_packages

from pkg_resources import get_distribution, DistributionNotFound


ROOT_PATH = os.path.dirname(os.path.realpath(__file__))
SOURCE_PATH = os.path.join(ROOT_PATH, 'source')
README_PATH = os.path.join(ROOT_PATH, 'README.rst')

try:
    release = get_distribution('covalent-python-api').version
    # take major/minor/patch
    VERSION = '.'.join(release.split('.')[:3])
except DistributionNotFound:
    # package is not installed
    VERSION = 'Unknown version'


version_template = '''
# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

__version__ = {version!r}
'''

# Call main setup.
setup(
    name='covalent-python-api',
    description='Python API for Covalent.',
    author='Lluis Casals Marsol',
    author_email='ktmlleska@gmail.com',
    url='https://bitbucket.org/lluisBrokenC/covalent_python_api',
    long_description=open(README_PATH).read(),
    license='GNU General Public License v3.0 (GPLv3)',
    keywords='covalent, python, api',
    packages=find_packages(SOURCE_PATH),
    project_urls={
        "Documentation": "https://www.covalenthq.com/docs/api/"
    },
    package_dir={
        '': 'source'
    },
    use_scm_version={
        'write_to': 'source/covalent_api/_version.py',
        'write_to_template': version_template,
    },
    setup_requires=[
        'docutils < 0.17',
        'sphinx >= 1.2.2, < 1.6',
        'sphinx_rtd_theme >= 0.1.6, < 1',
        'setuptools>=47.1.0',
        'setuptools_scm'
    ],
    install_requires=[
        'requests >= 2, <3',
    ],
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3'
    ],
    zip_safe=False,
    python_requires=">=3.7.6, <4.0"
)



# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

from types import FunctionType
import inspect


def get_class_methods(cls):
    return [
        x for x, y in cls.__dict__.items()
        if type(y) == FunctionType and not x.startswith('_')
    ]


def get_method_arguments(method):

    required_args = []
    optional_args = []
    for param_name, param_value in inspect.signature(method).parameters.items():
        if param_name != 'self':
            if param_value.default is param_value.empty:
                required_args.append(param_name)
            else:
                optional_args.append(param_name)
    return required_args, optional_args
